"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseBindingDefinition = parseBindingDefinition;
exports.createMapScopeToProps = createMapScopeToProps;
exports.wrapWithDecorators = wrapWithDecorators;

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseBindingDefinition(bindingDefinition) {
  const match = bindingDefinition.match(/^([@=<&])(\??)/);

  if (!match) {
    throw new Error("Defintion of binding '".concat(bindingDefinition, "' is not a valid"));
  }

  const binding = match[1];
  const optional = match[2] === '?'; // We only support '<' and '&'

  if (binding !== '<' && binding !== '&') {
    throw new Error("Binding '".concat(binding, "' is not a supported binding"));
  }

  return {
    binding,
    optional
  };
}

function createMapScopeToProps(bindings) {
  // We compile the details as early as possible because we want to validate the bindings before using the bindings.
  const bindingDetails = Object.keys(bindings).reduce(function (details, name) {
    details[name] = parseBindingDefinition(bindings[name]);
    return details;
  }, {});
  return function mapScopeToProps(scope) {
    return Object.keys(bindingDetails).reduce(function (props, propName) {
      const _bindingDetails$propN = bindingDetails[propName],
            binding = _bindingDetails$propN.binding,
            optional = _bindingDetails$propN.optional; // If the binding is optional and no value is passed we don't want to add it to the props

      if (optional && typeof scope[propName] === 'undefined') {
        return props;
      } // We expect that properties which are bound with & are functions


      if (binding === '&') {
        props[propName] = function (event) {
          return scope[propName]({
            $event: event
          });
        };
      } else {
        props[propName] = scope[propName];
      }

      return props;
    }, {});
  };
}

function wrapWithDecorators(decorators) {
  const compose = function (render, decorate) {
    // We create a new render function which takes props from outside
    const renderDecorator = function (props) {
      // Function which takes the passed props to render the component
      const renderComponentWithProps = function () {
        return render(props);
      }; // Pass the render function for component rendering to the decorate function
      // as render function


      return decorate(renderComponentWithProps);
    };

    return renderDecorator;
  };

  return function withDecorators(Component) {
    const render = function (props) {
      return _react.default.createElement(Component, props);
    };

    return decorators.reduce(compose, render);
  };
}